/**
  ******************************************************************************
  * @file    stm32xx_lpm.c
  * @author  MCD Application Team
  * @version V1.0.0
  * @date    01-October-2014
  * @brief   Low Power Manager
  ******************************************************************************
  * @attention
  *
  * <h2><center>&copy; COPYRIGHT(c) 2014 STMicroelectronics</center></h2>
  *
  * Redistribution and use in source and binary forms, with or without modification,
  * are permitted provided that the following conditions are met:
  *   1. Redistributions of source code must retain the above copyright notice,
  *      this list of conditions and the following disclaimer.
  *   2. Redistributions in binary form must reproduce the above copyright notice,
  *      this list of conditions and the following disclaimer in the documentation
  *      and/or other materials provided with the distribution.
  *   3. Neither the name of STMicroelectronics nor the names of its contributors
  *      may be used to endorse or promote products derived from this software
  *      without specific prior written permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
  * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
  * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
  ******************************************************************************
  */

/* Includes ------------------------------------------------------------------*/
#include "stm32xx_lpm_conf.h"

/* External variables --------------------------------------------------------*/
/* Private typedef -----------------------------------------------------------*/
/* Private macros ------------------------------------------------------------*/
/* Private defines -----------------------------------------------------------*/

/**
 * Masks to be used with the LPM_Mode_Request() API to read out the correct bit
 */
#define	LPM_DEEPSLEEP_MASK	0x04
#define	LPM_LPSDSR_MASK		0x02
#define	LPM_PDDS_MASK		0x01

/* Private macros ------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
static uint32_t uwDeepSleep_Req = 0;	/**< Default enabled - Control the CortexM3 DeepSleep bit */
static uint32_t uwLPSDSR_Req = 0;		/**< Default enabled - Control the LDO Eco Mode */
static uint32_t uwPDDS_Req = 0;			/**< Default enabled - Control the StandBy Mode */
static uint32_t uwRUN_Req = 0;			/**< Default disabled - RUN mode NOT requested */

/* Private function prototypes -----------------------------------------------*/
/* Public function prototypes -----------------------------------------------*/
WEAK void LPM_ExitStopMode(void);
WEAK void LPM_EnterStandbyMode(void);
WEAK void LPM_ExitStandbyMode(void);

/* Private functions ---------------------------------------------------------*/
/* Public functions ----------------------------------------------------------*/

/**
  * @brief  Interface to the user to request a specific low power mode
  *
  * @note
  *
  * @param  None
  *
  * @retval None
  */
void LPM_Mode_Request(eLPM_Id eId, eLPM_Mode eMode)
{
	uint32_t uwPRIMASK_Bit;

	uwPRIMASK_Bit = __get_PRIMASK();	/**< backup PRIMASK bit */
	__disable_irq();					/**< Disable all interrupts by setting PRIMASK bit on Cortex*/

	if(eMode == eLPM_Mode_RUN)			/**< RUN Mode request Management */
	{
		uwRUN_Req |= (uint32_t)(1<<eId);	/**< RUN mode enabled - set bit */
	}
	else
	{
		uwRUN_Req &= ~((uint32_t)(1<<eId));	/**< RUN mode Disable - clear bit */

		if(eMode&LPM_DEEPSLEEP_MASK)		/**< Register DeepSleep bit request */
		{
			uwDeepSleep_Req &= ~((uint32_t)(1<<eId));	/**< DeepSleep enabled - clear bit */
		}
		else
		{
			uwDeepSleep_Req |= (uint32_t)(1<<eId);
		}

		if(eMode&LPM_LPSDSR_MASK)	/**< Register LPSDSR bit request */
		{
			uwLPSDSR_Req &= ~((uint32_t)(1<<eId));	/**< LDO Eco Mode enabled - clear bit */
		}
		else
		{
			uwLPSDSR_Req |= (uint32_t)(1<<eId);
		}

		if(eMode&LPM_PDDS_MASK)	/**< Register PDDS bit request */
		{
			uwPDDS_Req &= ~((uint32_t)(1<<eId));	/**< StandBy Mode enabled - clear bit */
		}
		else
		{
			uwPDDS_Req |= (uint32_t)(1<<eId);
		}

		uwRUN_Req &= ~((uint32_t)(1<<eId));	/**< RUN mode disabled - clear bit */
	}

	__set_PRIMASK(uwPRIMASK_Bit);	/**< Restore PRIMASK bit*/

	return;
}
/**
  * @brief  Execute the low power mode configuration.
  *
  * @note	This function configures the MCU HW according to the lowest power mode requested by all FW modules.
  *			This function shall not be interrupted is executed in critical section using PRIMASK on the Cortex.
  *
  * @param  None
  *
  * @retval None
  */
void LPM_Enter_Mode(void)
{
	uint32_t uwPRIMASK_Bit;

	uwPRIMASK_Bit = __get_PRIMASK();	/**< backup PRIMASK bit */

	__disable_irq();	/**< Disable all interrupts by setting PRIMASK bit on Cortex*/

	if(uwRUN_Req == 0)
	{
		if(uwDeepSleep_Req)
		{
			/**
			 * Enter Sleep Mode
			 */
			HAL_PWR_EnterSLEEPMode(PWR_MAINREGULATOR_ON, PWR_SLEEPENTRY_WFI);
		}
		else if (uwPDDS_Req == 0)
		{
			LPM_EnterStandbyMode();	/**< Call user Hook */

			/**
			 * Clear Wakeup flag
			 */
			__HAL_PWR_CLEAR_FLAG(PWR_FLAG_WU);

			/**
			 * Enter Standby Mode
			 */
			HAL_PWR_EnterSTANDBYMode();

			LPM_ExitStandbyMode();	/**< Call user Hook */
		}
		else
		{
			/**
			 * Clear Wakeup flag
			 */
			__HAL_PWR_CLEAR_FLAG(PWR_FLAG_WU);

			/**
			 * Enter Stop Mode
			 */
			if(uwLPSDSR_Req)
			{
				HAL_PWR_EnterSTOPMode(PWR_MAINREGULATOR_ON, PWR_STOPENTRY_WFI);
			}
			else
			{
				HAL_PWR_EnterSTOPMode(PWR_LOWPOWERREGULATOR_ON, PWR_STOPENTRY_WFI);
			}

			LPM_ExitStopMode();	/**< Call user Hook */
		}
	}

	__set_PRIMASK(uwPRIMASK_Bit);	/**< Restore PRIMASK bit*/

    return;
}


/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/
