/**
  ******************************************************************************
  * @file    stm32xx_timerserver.c
  * @author  MCD Application Team
  * @version V1.0.0
  * @date    01-October-2014
  * @brief   Timer server
  ******************************************************************************
  * @attention
  *
  * <h2><center>&copy; COPYRIGHT(c) 2014 STMicroelectronics</center></h2>
  *
  * Redistribution and use in source and binary forms, with or without modification,
  * are permitted provided that the following conditions are met:
  *   1. Redistributions of source code must retain the above copyright notice,
  *      this list of conditions and the following disclaimer.
  *   2. Redistributions in binary form must reproduce the above copyright notice,
  *      this list of conditions and the following disclaimer in the documentation
  *      and/or other materials provided with the distribution.
  *   3. Neither the name of STMicroelectronics nor the names of its contributors
  *      may be used to endorse or promote products derived from this software
  *      without specific prior written permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
  * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
  * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
  ******************************************************************************
  */

/* Includes ------------------------------------------------------------------*/
#include "stm32xx_timerserver_conf.h"

/* External variables --------------------------------------------------------*/
/* Private typedef -----------------------------------------------------------*/
typedef enum{
	eTimerID_Free,
	eTimerID_Created,
	eTimerID_Running
}eTimerStatus_t;

typedef struct{
	pf_TIMER_TimerCallBack_t	pfTimerCallBack;
	uint16_t						uhCounterInit;
	uint16_t						uhCountLeft;
	eTimerStatus_t					eTimerStatus;
	eTIMER_TimerMode_t			eTimerMode;
	eTIMER_ModuleID_t			eTimerModuleID;
	uint8_t							ubPreviousID;
	uint8_t							ubNextID;
}sTimerContext_t;

/* Private macros ------------------------------------------------------------*/
/* Private defines -----------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
volatile sTimerContext_t aTimerContext[MAX_NBR_CONCURRENT_TIMER];
volatile uint8_t	ubCurrentRunningTimerID;
static RTC_HandleTypeDef *phrtc;	/**< RTC handle */

/* Private function prototypes -----------------------------------------------*/
static uint16_t GetCurrentCounter(void);

/* Private functions ---------------------------------------------------------*/

/**
  * @brief  Read the current wakeup counter
  *
  * @note	The API handles the corner case when the max value is read.
  * 		This could happen either because:
  * 		1) the timer did not yet start counting
  * 		2) The timer counted down to '0' and was reloaded automatically in which case the value to consider is '0'
  *
  * @param  None
  *
  * @retval Return the value read from the wakeup counter
  */
static uint16_t GetCurrentCounter(void)
{
	uint16_t uhCurrentCounter;

	uhCurrentCounter = 0;

	while((uhCurrentCounter != HAL_RTCEx_GetWakeUpTimer(phrtc)) && (__HAL_RTC_WAKEUPTIMER_GET_FLAG(phrtc, RTC_FLAG_WUTF) == RESET))
	{
		uhCurrentCounter = HAL_RTCEx_GetWakeUpTimer(phrtc);
	}

	if(__HAL_RTC_WAKEUPTIMER_GET_FLAG(phrtc, RTC_FLAG_WUTF) == SET)
	{
		uhCurrentCounter = 0;
	}

	return uhCurrentCounter;
}

/* Public functions ----------------------------------------------------------*/

/**
  * @brief  Schedule the timer list on the timer interrupt handler
  *
  * @note
  *
  * @param  None
  *
  * @retval None
  */
void TIMER_RTC_Wakeup_Handler(void)
{
	pf_TIMER_TimerCallBack_t pfTimerCallBack;
	eTIMER_ModuleID_t eTimerModuleID;
	uint16_t localCountLeft;
	uint8_t	localCurrentRunningTimerID;
	uint8_t	localNextID;
	uint8_t	ubTimerIDLookUp;

    /**
     * Disable the Wakeup Timer
     */
    __HAL_RTC_WAKEUPTIMER_DISABLE(phrtc);

	/**
	 * Clear flag in EXTI module
	 */
    __HAL_RTC_EXTI_CLEAR_FLAG(RTC_EXTI_LINE_WAKEUPTIMER_EVENT);

	localCurrentRunningTimerID = ubCurrentRunningTimerID;

	if(aTimerContext[localCurrentRunningTimerID].eTimerStatus == eTimerID_Running)
	{
		/**
		 * update count for all timers left
		 */
		ubTimerIDLookUp = localCurrentRunningTimerID;
		while((localNextID = aTimerContext[ubTimerIDLookUp].ubNextID) != MAX_NBR_CONCURRENT_TIMER)
		{
			localCountLeft = aTimerContext[localCurrentRunningTimerID].uhCountLeft;
			aTimerContext[localNextID].uhCountLeft -= localCountLeft;
			ubTimerIDLookUp = localNextID;
		}

		aTimerContext[localCurrentRunningTimerID].uhCountLeft = 0;

		pfTimerCallBack = aTimerContext[localCurrentRunningTimerID].pfTimerCallBack;
		eTimerModuleID = aTimerContext[localCurrentRunningTimerID].eTimerModuleID;

		if(aTimerContext[localCurrentRunningTimerID].eTimerMode == eTimerMode_Repeated)
		{
			TIMER_Start(localCurrentRunningTimerID, aTimerContext[localCurrentRunningTimerID].uhCounterInit);

			/**
			 * Clear flag in RTC module AFTER TIMER_Start() is called
			 * the RTC flag is checked when reading the wakeup timer to readout the correct value
			 * It shall not be cleared before the wakeup counter has been read
			 */
			__HAL_RTC_WAKEUPTIMER_CLEAR_FLAG(phrtc, RTC_FLAG_WUTF);

			/**
			 * The timer may have been linked after the current timer running. The TIMER_Start() API is not aware the
			 * timer has been stopped in the handler so there should be a checked and a restart in case the timer is stopped			 *
			 */
			if( __HAL_TIMERSERVER_RTC_GET_TIMER_STATUS(phrtc) == RESET)
			{
				while(__HAL_RTC_WAKEUPTIMER_GET_FLAG(phrtc, RTC_FLAG_WUTWF) == RESET);

				/**
				 * Write next count
				 */
				__HAL_TIMERSERVER_RTC_SET_WAKEUPCOUNTER(phrtc, aTimerContext[ubCurrentRunningTimerID].uhCountLeft);

				/**
				 * Enable the Wakeup Timer
				 */
				__HAL_RTC_WAKEUPTIMER_ENABLE(phrtc);
			}
		}
		else
		{
			TIMER_Stop(localCurrentRunningTimerID);

			/**
			 * Clear flag in RTC module AFTER TIMER_Stop(() is called
			 * the RTC flag is checked when reading the wakeup timer to readout the correct value
			 * It shall not be cleared before the wakeup counter has been read
			 */
			__HAL_RTC_WAKEUPTIMER_CLEAR_FLAG(phrtc, RTC_FLAG_WUTF);
		}

		TIMER_Notification(eTimerModuleID, pfTimerCallBack);
	}
	else
	{
		__HAL_RTC_WAKEUPTIMER_CLEAR_FLAG(phrtc, RTC_FLAG_WUTF);
	}

	return;
}

/**
  * @brief  Initialize the timer server
  *
  * @note
  *
  * @param  RTC Handle
  *
  * @retval None
  */
void TIMER_Init(RTC_HandleTypeDef *hrtc)
{
	uint8_t	ubLoop;

	/**
	 * Get RTC handler
	 */
	phrtc = hrtc;

	/**
	 * Reinit everything for the scope of this projet
	 */
	__HAL_RTC_WAKEUPTIMER_DISABLE(phrtc);						/**<  Disable the Wakeup Timer */
    __HAL_RTC_WAKEUPTIMER_CLEAR_FLAG(phrtc, RTC_FLAG_WUTF);		/**<  Clear flag in RTC module */
    __HAL_RTC_EXTI_CLEAR_FLAG(RTC_EXTI_LINE_WAKEUPTIMER_EVENT);	/**<  Clear flag in EXTI module  */
    HAL_NVIC_ClearPendingIRQ(NVIC_RTC_WAKEUP_HANDLER_ID);			/**<  Clear pending bit in NVIC  */
	__HAL_RTC_WAKEUPTIMER_ENABLE_IT(phrtc, RTC_IT_WUT);			/**<  Enable interrupt in RTC module  */

	/**
	 * Configure EXTI module
	 */
	__HAL_TIMERSERVER_EXTI_RTC_SET_WAKEUPTIMER_RISING_EDGE();
	__HAL_RTC_EXTI_ENABLE_IT(RTC_EXTI_LINE_WAKEUPTIMER_EVENT);


	/**
	 * Initialize the timer server
	 */
	for(ubLoop = 0; ubLoop < MAX_NBR_CONCURRENT_TIMER; ubLoop++)
	{
		aTimerContext[ubLoop].eTimerStatus = eTimerID_Free;
	}

	ubCurrentRunningTimerID = MAX_NBR_CONCURRENT_TIMER;		/**<  Set ID to non valid value */
	HAL_NVIC_SetPriority(NVIC_RTC_WAKEUP_HANDLER_ID, NVIC_UART_RTC_WAKEUP_IT_PREEMPTPRIORITY, NVIC_UART_RTC_WAKEUP_IT_SUBPRIORITY);		/**<  Set NVIC priority */

	return;
}

/**
  * @brief  Interface to create a virtual timer
  *
  * @note
  *
  * @param  eTimerModuleID:	This is an identifier provided by the user and returned in the callback to allow
  * 						identification of the requester
  *
  * @param  pTimerId:		Timer Id returned to the user to request operation (start, stop, delete)
  *
  * @param  eTimerMode:		Mode of the virtual timer (Single shot or repeated)
  *
  * @param  pftimeout_handler:	Callback when the virtual timer expires
  *
  * @retval None
  */
void TIMER_Create(eTIMER_ModuleID_t eTimerModuleID, uint8_t *pTimerId, eTIMER_TimerMode_t eTimerMode, pf_TIMER_TimerCallBack_t pftimeout_handler)
{
	uint8_t	ubLoop = 0;

	TIMER_ENTER_CRITICAL_SECTION;
	while(aTimerContext[ubLoop].eTimerStatus != eTimerID_Free)
	{
		ubLoop++;
	}

	aTimerContext[ubLoop].eTimerStatus = eTimerID_Created;
	TIMER_EXIT_CRITICAL_SECTION	;

	aTimerContext[ubLoop].eTimerModuleID = eTimerModuleID;
	aTimerContext[ubLoop].eTimerMode = eTimerMode;
	aTimerContext[ubLoop].pfTimerCallBack = pftimeout_handler;
	*pTimerId = ubLoop;

	return;
}

/**
  * @brief  Delete a virtual timer from the list
  *
  * @note	If the virtual timer is running, it is stopped and deleted.
  *
  * @param  ubTimerID:	Id of the timer to remove from the list
  *
  * @retval None
  */
void TIMER_Delete(uint8_t ubTimerID)
{
	TIMER_Stop(ubTimerID);

	aTimerContext[ubTimerID].eTimerStatus = eTimerID_Free;	/**<  release ID */

	return;
}

/**
  * @brief  Stop a virtual timer
  *
  * @note	A virtual timer that has been stopped may be restarted
  *
  * @param  ubTimerID:	Id of the timer to stop
  *
  * @retval None
  */
void TIMER_Stop(uint8_t ubTimerID)
{
	uint16_t	uhTimeElapsed;
	uint8_t		ubTimerIDLookUp;
	uint8_t		localID;

	TIMER_ENTER_CRITICAL_SECTION;

	HAL_NVIC_DisableIRQ(NVIC_RTC_WAKEUP_HANDLER_ID);		/**<  Disable NVIC */

	if(aTimerContext[ubTimerID].eTimerStatus == eTimerID_Running)
	{
		/**
		 * Update current running timer ID if needed
		 */
		if(ubTimerID == ubCurrentRunningTimerID)
		{
			if(aTimerContext[ubTimerID].ubNextID != MAX_NBR_CONCURRENT_TIMER)
			{
				uhTimeElapsed = aTimerContext[ubTimerID].uhCountLeft - GetCurrentCounter();

				__HAL_RTC_WAKEUPTIMER_DISABLE(phrtc);						/**<  Disable the Wakeup Timer */
				__HAL_RTC_WAKEUPTIMER_CLEAR_FLAG(phrtc, RTC_FLAG_WUTF);		/**<  Clear flag in RTC module */
				__HAL_RTC_EXTI_CLEAR_FLAG(RTC_EXTI_LINE_WAKEUPTIMER_EVENT);	/**<  Clear flag in EXTI module */
				HAL_NVIC_ClearPendingIRQ(NVIC_RTC_WAKEUP_HANDLER_ID);			/**<  Clear pending bit in NVIC */

				/**
				 * update all counts
				 */
				ubTimerIDLookUp = ubTimerID;
				while((localID = aTimerContext[ubTimerIDLookUp].ubNextID) != MAX_NBR_CONCURRENT_TIMER)
				{
					aTimerContext[localID].uhCountLeft -= uhTimeElapsed;
					ubTimerIDLookUp = aTimerContext[ubTimerIDLookUp].ubNextID;
				}

				while(__HAL_RTC_WAKEUPTIMER_GET_FLAG(phrtc, RTC_FLAG_WUTWF) == RESET);

				/**
				 * Write next count
				 */
				__HAL_TIMERSERVER_RTC_SET_WAKEUPCOUNTER(phrtc, (aTimerContext[aTimerContext[ubTimerID].ubNextID].uhCountLeft - uhTimeElapsed));

				__HAL_RTC_WAKEUPTIMER_ENABLE(phrtc);	/**<  Enable the Wakeup Timer */
			}
			else
			{
				__HAL_RTC_WAKEUPTIMER_DISABLE(phrtc);						/**<  Disable the Wakeup Timer */
				__HAL_RTC_WAKEUPTIMER_CLEAR_FLAG(phrtc, RTC_FLAG_WUTF);		/**<  Clear flag in RTC module */
				__HAL_RTC_EXTI_CLEAR_FLAG(RTC_EXTI_LINE_WAKEUPTIMER_EVENT);	/**<  Clear flag in EXTI module */
				HAL_NVIC_ClearPendingIRQ(NVIC_RTC_WAKEUP_HANDLER_ID);			/**<  Clear pending bit in NVIC */
			}

			ubCurrentRunningTimerID = aTimerContext[ubTimerID].ubNextID;
		}
		else
		{
			/**
			 * unlink timer
			 */
			localID = aTimerContext[ubTimerID].ubPreviousID;
			aTimerContext[localID].ubNextID = aTimerContext[ubTimerID].ubNextID;
			if((localID = aTimerContext[ubTimerID].ubNextID) != MAX_NBR_CONCURRENT_TIMER)
			{
				aTimerContext[localID].ubPreviousID = aTimerContext[ubTimerID].ubPreviousID;
			}
		}
	}

	/**
	 * Timer is out of the list
	 */
	aTimerContext[ubTimerID].eTimerStatus = eTimerID_Created;

	HAL_NVIC_EnableIRQ(NVIC_RTC_WAKEUP_HANDLER_ID);	/**<  Enable NVIC */

	TIMER_EXIT_CRITICAL_SECTION;

	return;
}

/**
  * @brief  Start a virtual timer
  *
  * @note	The value of the ticks is depending on the user RTC clock input configuration
  *
  * @param  ubTimerID:	Id of the timer to star
  *
  * @param  uhTimeoutTicks:	Number of ticks of the virtual timer
  *
  * @retval None
  */
void TIMER_Start(uint8_t ubTimerID, uint16_t uhTimeoutTicks)
{
	uint16_t	uhTimeElapsed;
	uint8_t	ubTimerIDLookUp;
	uint8_t localNextID;

	TIMER_ENTER_CRITICAL_SECTION;

	HAL_NVIC_DisableIRQ(NVIC_RTC_WAKEUP_HANDLER_ID);		/**<  Disable NVIC */

	aTimerContext[ubTimerID].eTimerStatus = eTimerID_Running;

	if(ubCurrentRunningTimerID == MAX_NBR_CONCURRENT_TIMER)
	{
		/**
		 * No timer in the list
		 */
		ubCurrentRunningTimerID = ubTimerID;
		aTimerContext[ubTimerID].ubNextID = MAX_NBR_CONCURRENT_TIMER;

		while(__HAL_RTC_WAKEUPTIMER_GET_FLAG(phrtc, RTC_FLAG_WUTWF) == RESET);

		__HAL_TIMERSERVER_RTC_SET_WAKEUPCOUNTER(phrtc, uhTimeoutTicks);	/**<  Write next count */
		__HAL_RTC_WAKEUPTIMER_ENABLE(phrtc);						/**<  Enable the Wakeup Timer */
	}
	else
	{
		uhTimeElapsed = aTimerContext[ubCurrentRunningTimerID].uhCountLeft - GetCurrentCounter();

		ubTimerIDLookUp = ubCurrentRunningTimerID;
		/**
		 * Link the timer
		 */
		while( (aTimerContext[ubTimerIDLookUp].ubNextID != MAX_NBR_CONCURRENT_TIMER) && ((aTimerContext[ubTimerIDLookUp].uhCountLeft-uhTimeElapsed) < uhTimeoutTicks) )
		{
			ubTimerIDLookUp = aTimerContext[ubTimerIDLookUp].ubNextID;
		}

		/**
		 * ubTimerID is either juste BEFORE or just AFTER ubTimerIDLookUp
		 */

		if((aTimerContext[ubTimerIDLookUp].uhCountLeft-uhTimeElapsed) < uhTimeoutTicks)
		{
			/**
			 * Link timer AFTER ubTimerIDLookUp
			 */
			if( (ubTimerID == ubCurrentRunningTimerID) && (aTimerContext[ubCurrentRunningTimerID].ubNextID != MAX_NBR_CONCURRENT_TIMER) )
			{
				ubCurrentRunningTimerID = aTimerContext[ubCurrentRunningTimerID].ubNextID;
			}

			if(ubTimerIDLookUp != ubTimerID)
			{
				if(aTimerContext[ubTimerIDLookUp].ubNextID != MAX_NBR_CONCURRENT_TIMER)
				{
					aTimerContext[aTimerContext[ubTimerIDLookUp].ubNextID].ubPreviousID = ubTimerID;
				}

				aTimerContext[ubTimerID].ubNextID = aTimerContext[ubTimerIDLookUp].ubNextID;
				aTimerContext[ubTimerID].ubPreviousID = ubTimerIDLookUp ;
				aTimerContext[ubTimerIDLookUp].ubNextID = ubTimerID;
			}
		}
		else
		{
			/**
			 * Link timer BEFORE ubTimerIDLookUp
			 */

			if(ubTimerIDLookUp != ubCurrentRunningTimerID)
			{
				if( (aTimerContext[ubCurrentRunningTimerID].ubNextID != ubTimerIDLookUp) && (ubCurrentRunningTimerID == ubTimerID) )
				{
					ubCurrentRunningTimerID = aTimerContext[ubCurrentRunningTimerID].ubNextID;

					aTimerContext[aTimerContext[ubTimerIDLookUp].ubPreviousID].ubNextID = ubTimerID;
					aTimerContext[ubTimerID].ubPreviousID = aTimerContext[ubTimerIDLookUp].ubPreviousID ;

					aTimerContext[ubTimerID].ubNextID = ubTimerIDLookUp;
					aTimerContext[ubTimerIDLookUp].ubPreviousID = ubTimerID;

					while(__HAL_RTC_WAKEUPTIMER_GET_FLAG(phrtc, RTC_FLAG_WUTWF) == RESET);

					__HAL_TIMERSERVER_RTC_SET_WAKEUPCOUNTER(phrtc, aTimerContext[ubCurrentRunningTimerID].uhCountLeft);	/**<  Write next count */
					__HAL_RTC_WAKEUPTIMER_ENABLE(phrtc);							/**<  Enable the Wakeup Timer */
				}
				else
				{
					aTimerContext[aTimerContext[ubTimerIDLookUp].ubPreviousID].ubNextID = ubTimerID;
					aTimerContext[ubTimerID].ubPreviousID = aTimerContext[ubTimerIDLookUp].ubPreviousID ;

					aTimerContext[ubTimerID].ubNextID = ubTimerIDLookUp;
					aTimerContext[ubTimerIDLookUp].ubPreviousID = ubTimerID;
				}
			}
			else
			{
				if(ubTimerIDLookUp != ubTimerID)
				{
					aTimerContext[ubTimerID].ubNextID = ubTimerIDLookUp;
					aTimerContext[ubTimerIDLookUp].ubPreviousID = ubTimerID;
				}

				__HAL_RTC_WAKEUPTIMER_DISABLE(phrtc);						/**<  Disable the Wakeup Timer */
				__HAL_RTC_WAKEUPTIMER_CLEAR_FLAG(phrtc, RTC_FLAG_WUTF);		/**<  Clear flag in RTC module */
				__HAL_RTC_EXTI_CLEAR_FLAG(RTC_EXTI_LINE_WAKEUPTIMER_EVENT);	/**<  Clear flag in EXTI module */
				HAL_NVIC_ClearPendingIRQ(NVIC_RTC_WAKEUP_HANDLER_ID);			/**<  Clear pending bit in NVIC */

				ubCurrentRunningTimerID = ubTimerID;

				while(__HAL_RTC_WAKEUPTIMER_GET_FLAG(phrtc, RTC_FLAG_WUTWF) == RESET);

				__HAL_TIMERSERVER_RTC_SET_WAKEUPCOUNTER(phrtc, uhTimeoutTicks);	/**<  Write next count */
				__HAL_RTC_WAKEUPTIMER_ENABLE(phrtc);						/**<  Enable the Wakeup Timer */

			    /**
			     * update all counts
			     */
				ubTimerIDLookUp = ubCurrentRunningTimerID;
				while((localNextID = aTimerContext[ubTimerIDLookUp].ubNextID) != MAX_NBR_CONCURRENT_TIMER)
				{
					aTimerContext[localNextID].uhCountLeft -= uhTimeElapsed;
					ubTimerIDLookUp = aTimerContext[ubTimerIDLookUp].ubNextID;
				}
			}
		}
	}

	aTimerContext[ubTimerID].uhCountLeft = uhTimeoutTicks;
	aTimerContext[ubTimerID].uhCounterInit = uhTimeoutTicks;

	HAL_NVIC_EnableIRQ(NVIC_RTC_WAKEUP_HANDLER_ID);	/**<  Enable NVIC */

	TIMER_EXIT_CRITICAL_SECTION;

	return;
}

/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/
