/**
  ******************************************************************************
  * @file    blocking.c
  * @author  MCD Application Team
  * @version V1.0.0
  * @date    01-October-2014
  * @brief   Pausing BLE between Command and Response
  ******************************************************************************
  * @attention
  *
  * <h2><center>&copy; COPYRIGHT(c) 2014 STMicroelectronics</center></h2>
  *
  * Redistribution and use in source and binary forms, with or without modification,
  * are permitted provided that the following conditions are met:
  *   1. Redistributions of source code must retain the above copyright notice,
  *      this list of conditions and the following disclaimer.
  *   2. Redistributions in binary form must reproduce the above copyright notice,
  *      this list of conditions and the following disclaimer in the documentation
  *      and/or other materials provided with the distribution.
  *   3. Neither the name of STMicroelectronics nor the names of its contributors
  *      may be used to endorse or promote products derived from this software
  *      without specific prior written permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
  * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
  * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
  ******************************************************************************
  */

/* Includes ------------------------------------------------------------------*/
#include "blocking.h"

#include <stdint.h>
#include <stdbool.h>

#include "ble_encode_access.h"
#include "main.h"

#if (APP_HRS == 1)
#include "ble_app_main.h"
#endif

/* Private variables ---------------------------------------------------------*/
static volatile bool        m_cmd_rsp_event_rcvd = false;
static uint32_t             m_cmd_result_code;

/* Private function prototypes -----------------------------------------------*/
static void internal_response_handler(uint32_t result_code);

/* Private functions ---------------------------------------------------------*/

/**
  * @brief  Handler to notifying a response packet has been received
  *
  * @note	This API is called when a response packet has been received
  *
  * @param  result_code: Error code while decoding the response packet
  *
  * @retval None
  */
static void internal_response_handler(uint32_t result_code)
{
    m_cmd_result_code = result_code;
    m_cmd_rsp_event_rcvd = true;

    /**
     * This is added to solve race condition when this event occurs between the check of the variable m_cmd_rsp_event_rcvd
     * and the time we enter low power mode
     * This will prevent entering low power mode and will force the re-evaluation of the variable m_cmd_rsp_event_rcvd
     */
    TaskExecutionRequest(eMAIN_Main_SD_Command_Resp);

    return;
}

/* Public functions ----------------------------------------------------------*/

/**
  * @brief  Initialization of the blocking mechanism to prevent sending two SD command back to back
  *
  * @note
  *
  * @param  None
  *
  * @retval None
  */
uint32_t blocking_init(void)
{
    ble_encode_cmd_resp_handler_reg(internal_response_handler);
    
    return NRF_SUCCESS;
}

/**
  * @brief  Interface to pause the BLE module until a response packet is received
  *
  * @note	This API is called after each SD command sent to the nRF device.
  *
  * @param  None
  *
  * @retval None
  */
uint32_t blocking_resp_wait(void)
{
    while (!m_cmd_rsp_event_rcvd)
    {
    	Background(SD_COMMAND_NOT_ALLOWED);
    }
    
    m_cmd_rsp_event_rcvd = false;
    TaskExecuted(eMAIN_Main_SD_Command_Resp);
    
    return m_cmd_result_code;
}

/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/
