/**
  ******************************************************************************
  * @file    main.c
  * @author  MCD Application Team
  * @version V1.0.0
  * @date    01-October-2014
  * @brief   Main body
  ******************************************************************************
  * @attention
  *
  * <h2><center>&copy; COPYRIGHT(c) 2014 STMicroelectronics</center></h2>
  *
  * Redistribution and use in source and binary forms, with or without modification,
  * are permitted provided that the following conditions are met:
  *   1. Redistributions of source code must retain the above copyright notice,
  *      this list of conditions and the following disclaimer.
  *   2. Redistributions in binary form must reproduce the above copyright notice,
  *      this list of conditions and the following disclaimer in the documentation
  *      and/or other materials provided with the distribution.
  *   3. Neither the name of STMicroelectronics nor the names of its contributors
  *      may be used to endorse or promote products derived from this software
  *      without specific prior written permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
  * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
  * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
  ******************************************************************************
  */

/* Includes ------------------------------------------------------------------*/
#include "main.h"
#include "app_conf.h"
#include "ble_app_main.h"
#include "msp_main.h"
#include "stm32l0xx_nucleo.h"
#include "wavetek_nRF.h"
#include "stm32l0xx_hal_app_rcc.h"

#if(APP_L2CAP_TX_TEST == 1)
#include "ble_l2cap.h"
#include "ble.h"
#include "blocking.h"
#endif

/* External variables --------------------------------------------------------*/
pf_Main_UserButtonHandler_t pf_UserButtonHandler;

/* Private typedef -----------------------------------------------------------*/
/* Private defines -----------------------------------------------------------*/
#define CONN_CHIP_RESET_TIME		5	/**< The time to keep the reset line to the nRF51822 low ( 2 milliseconds). */
#define CONN_CHIP_STARTUP_TIME		19  /**< Wait for the nRF device to be ready before sending any command ( 7.5 milliseconds). */

#if(APP_L2CAP_TX_TEST == 1)
#define L2CAP_HEADER_CID			0x0044		/**< L2CAP header CID */
#define L2CAP_PAYLOAD_LENGTH		23			/**< Length of the L2CAP payload */
#define L2CAP_PAYLOAD_PATTERN		0x55		/**< Pattern to be sent in the payload */
#endif

#if (APP_LEDBUTTON == 1)
#define BUTTON_1					0x01		/**< Value to send when Button 1 is pushed */
#define BUTTON_2					0x02    	/**< Value to send when Button 2 is pushed */
#define BUTTON_3					0x03    	/**< Value to send when Button 3 is pushed */

#define LED1_VALUE					0x01
#define LED2_VALUE					0x02
#define LED3_VALUE					0x04
#endif
/* Private macros ------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
static uint32_t TaskExecutionRequested = 0;
static volatile uint8_t	ubnRFresetTimerLock;
pf_TIMER_TimerCallBack_t pfTimerBLECallBack;
static RTC_HandleTypeDef hrtc;	/**< RTC handle */

#if(APP_HTS == 1)
volatile int16_t RawTemperatureData;
int16_t CalibratedTemperatureData;
ADC_HandleTypeDef	hadc;
#endif


#if(APP_L2CAP_TX_TEST == 1)
static uint8_t	FreeBufferCount;		/**< Number of buffer available for L2CAP transfer on nRF side */
uint8_t L2CAPPayloadBuffer[L2CAP_PAYLOAD_LENGTH];
ble_l2cap_header_t l2cap_header;
#endif


/* Private function prototypes -----------------------------------------------*/
static void pf_nRFResetTimerCallBack(void);
static void Init_RTC(void);
static void connectivity_chip_reset(void);
#if(APP_HTS == 1)
static void ProcessADCdata(void);
#endif

/* Private functions ---------------------------------------------------------*/

/**
  * @brief  This function notify when then nRF51 nRESET may be released
  *
  * @note
  *
  * @param  None
  *
  * @retval None
  */
static void pf_nRFResetTimerCallBack(void)
{
	ubnRFresetTimerLock = 0;

	return;
}


#if(APP_HTS == 1)
/**
  * @brief  This function processes the data received from ADC
  *
  * @note
  *
  * @param  None
  *
  * @retval None
  */
void ProcessADCdata(void)
{
	CalibratedTemperatureData = CalculateCalibratedTemperature((int16_t*)&RawTemperatureData);
	LPM_Mode_Request(eLPM_ADC_Temp_Measurement, eLPM_Mode_StandBy);
	temperature_measurement_send();

	return;
}
#endif

/**
  * @brief  Initialize RTC block
  *
  * @note
  *
  * @param  None
  *
  * @retval None
  */
static void Init_RTC(void)
{

	/* Initialize the HW - 37Khz LSI being used*/
	/* Enable the LSI clock */
	__HAL_RCC_LSI_ENABLE();

	/* Enable power module clock */
	__PWR_CLK_ENABLE();

	/* Enable acces to the RTC registers */
	HAL_PWR_EnableBkUpAccess();

	/* Select LSI as RTC Input */
	__HAL_RCC_RTC_CONFIG(RCC_RTCCLKSOURCE_LSI);

	/* Enable RTC */
	__HAL_RCC_RTC_ENABLE();

	hrtc.Instance = RTC;				/**< Define instance */
	hrtc.Lock = HAL_UNLOCKED;			/**< Initialize lock */
	hrtc.State = HAL_RTC_STATE_READY;	/**< Initialize state */

	/* Disable Write Protection */
	__HAL_RTC_WRITEPROTECTION_DISABLE(&hrtc) ;

	/* Wait for LSI to be stable */
	while(__HAL_RCC_GET_FLAG(RCC_FLAG_LSIRDY) == 0);

	return;
}

/**
  * @brief  Function for resetting the nRF51 with IO PC7.
  *
  * @note
  *
  * @param  None
  *
  * @retval None
  */
static void connectivity_chip_reset(void)
{
	GPIO_InitTypeDef GPIO_InitStructure;
	uint8_t	ubnRFResetTimerID;

	__GPIOC_CLK_ENABLE();	/**< Switch ON GPIO AHB clock */

	HAL_GPIO_WritePin(GPIOC, GPIO_PIN_7, GPIO_PIN_RESET);	/**< Drive low the GPIO PC7 to reset nRF51 */
  	GPIO_InitStructure.Mode = GPIO_MODE_OUTPUT_PP;
  	GPIO_InitStructure.Pull = GPIO_NOPULL;
  	GPIO_InitStructure.Pin = GPIO_PIN_7;
  	GPIO_InitStructure.Speed = GPIO_SPEED_HIGH;
  	HAL_GPIO_Init(GPIOC, &GPIO_InitStructure);


    TIMER_Create(eTimerModuleID_Interrupt, &ubnRFResetTimerID, eTimerMode_SingleShot, pf_nRFResetTimerCallBack);
    TIMER_Start(ubnRFResetTimerID, CONN_CHIP_RESET_TIME);
    ubnRFresetTimerLock = 1;
    while(ubnRFresetTimerLock == 1);

	/*
	 * Set IO pin in Analog mode
	 */
  	GPIO_InitStructure.Mode = GPIO_MODE_ANALOG;
  	GPIO_InitStructure.Pull = GPIO_NOPULL;
  	GPIO_InitStructure.Pin = GPIO_PIN_7;
  	HAL_GPIO_Init(GPIOC, &GPIO_InitStructure);

  	__GPIOC_CLK_DISABLE();	/**< Switch OFF GPIO AHB clock */

	/*
	 * Wait for the nRF device to be ready before sending any command
	 */
    TIMER_Start(ubnRFResetTimerID, CONN_CHIP_STARTUP_TIME);
    ubnRFresetTimerLock = 1;
    while(ubnRFresetTimerLock == 1);
    TIMER_Delete(ubnRFResetTimerID);


	return;
}

/* Public functions ----------------------------------------------------------*/

/**
  * @brief  NVM Low power mode request
  *
  * @note
  *
  * @param  eHSclkMode: Mode requested
  *
  * @retval None
  */
void NVM_HSclkRequest(eNVM_HSclkMode_t eHSclkMode)
{
	if(eHSclkMode == eNVM_HSclkEnable)
	{
		LPM_Mode_Request(eLPM_NVM, eLPM_Mode_Sleep);
	}
	else
	{
		LPM_Mode_Request(eLPM_NVM, eLPM_Mode_StandBy);
	}

	return;
}

/**
  * @brief  UART Low power mode request
  *
  * @note
  *
  * @param  eHSclkRequester: Requester Id
  *
  * @param  eHSclkMode: Mode requested
  *
  * @retval None
  */
void LPPUART_HSclkRequest(eLPPUART_HSclkRequester_t eHSclkRequester, eLPPUART_HSclkMode_t eHSclkMode)
{
	if(eHSclkMode == eLPPUART_HSclkEnable)
	{
		LPM_Mode_Request((eLPM_Id) eHSclkRequester, eLPM_Mode_Sleep);
	}
	else
	{
		LPM_Mode_Request((eLPM_Id) eHSclkRequester, eLPM_Mode_StandBy);
	}

	return;
}


/**
  * @brief  Function for application main entry.
  *
  * @note
  *
  * @param  None
  *
  * @retval None
  */
int main(void)
{
#if(APP_L2CAP_TX_TEST == 1)
	uint8_t loop;
#endif

	/*
	 * The APB power block clock is enable during the startup phase
	 */

#if (JTAG_SUPPORTED == 1)
	/*
	 * Keep debugger enable while in any low power mode
	 */
	__DBGMCU_CLK_ENABLE();
	HAL_DBG_LowPowerConfig(DBGMCU_SLEEP | DBGMCU_STOP | DBGMCU_STANDBY, ENABLE);
#endif

	/**
	 *  HAL library initialization:
	 *    - Configure the Flash prefetch, Flash preread and Buffer caches
	 *    - Systick timer is configured by default as source of time base, but user
	 *          can eventually implement his proper time base source (a general purpose
	 *          timer for example or other time source), keeping in mind that Time base
	 *          duration should be kept 1ms since PPP_TIMEOUT_VALUEs are defined and
	 *          handled in milliseconds basis.
	 *    - Low Level Initialization
	 */
	HAL_Init();

	/*
	 * System clock initialisation
	 */
	SystemClock_Config();

	__HAL_PWR_CLEAR_FLAG(PWR_FLAG_SB);	/**< Clear Standby flag */
	__HAL_RCC_CLEAR_RESET_FLAGS();		/**< clear all reset flags */

	SystemPower_Config();

	BSP_PB_Init(BUTTON_KEY, BUTTON_MODE_EXTI);	/**< user button */

	UserButtonHandlerRegister((pf_Main_UserButtonHandler_t)user_nvm_bond_delete); /**< user button callback */
#if (APP_LEDBUTTON == 1)
	/**
	 * initialize the LED
	 */
	wavetek_nRF_led_init(Wavetek_nRF_LED1_Id);
	wavetek_nRF_led_init(Wavetek_nRF_LED2_Id);
	wavetek_nRF_led_init(Wavetek_nRF_LED3_Id);

	/**
	 * initialize the BUTTON
	 */
	wavetek_nRF_button_init(Wavetek_nRF_BUTTON1_Id);
	wavetek_nRF_button_init(Wavetek_nRF_BUTTON2_Id);
	wavetek_nRF_button_init(Wavetek_nRF_BUTTON3_Id);
#endif

#if (APP_HTS == 1)
	/*
	 * Configure Temp Acquisition
	 */
	InitADC_TemperatureSensor(&hadc);	/**< Configure ADC for temperature sensor value conversion */
#endif
	/********************************************************************/

	LPM_Mode_Request(eLPM_Application, eLPM_Mode_LP_Stop);	/**< Define minimum low power mode allowed in the system */

	Init_RTC();
    TIMER_Init(&hrtc);
    NVM_Init();

	connectivity_chip_reset();	/**< Reset connectivity chip. */

	UART_BLE_Init();			/**< Request all resources for the UART communication with the nRF device */

	ble_app_main_init_Reset();	/**< Initialize connectivity chip. */

	LPM_Mode_Request(eLPM_Application, LOWEST_LOW_POWER_MODE);	/**< Define minimum low power mode allowed in the system */

#if(APP_L2CAP_TX_TEST == 1)
	/*
	 * Fill in payload buffer
	 */
	for(loop = 0; loop < L2CAP_PAYLOAD_LENGTH; loop++)
	{
		L2CAPPayloadBuffer[loop] = L2CAP_PAYLOAD_PATTERN;
	}

	/*
	 * Initialize L2CAP header
	 */
    l2cap_header.len = L2CAP_PAYLOAD_LENGTH;
	l2cap_header.cid = L2CAP_HEADER_CID;

	/*
	 * Get max buffer available for L2CAP data on nRF device
	 */
	 sd_ble_tx_buffer_count_get(&FreeBufferCount);
	 blocking_resp_wait();
#endif

	/*
	 * Enter main loop.
	 */
    for (;;)
    {
    	Background(SD_COMMAND_ALLOWED);
	}
}

/**
  * @brief  Request action to the scheduler in background
  *
  * @note
  *
  * @param  eMAIN_Backround_Task_Id: Id of the request
  *
  * @retval None
  */
void TaskExecutionRequest(eMAIN_Backround_Task_Id_t eMAIN_Backround_Task_Id)
{
	__disable_irq();
	TaskExecutionRequested |= (1 << eMAIN_Backround_Task_Id);
	__enable_irq();

	return;
}

/**
  * @brief  Notify the action in background has been completed
  *
  * @note
  *
  * @param  eMAIN_Backround_Task_Id: Id of the request
  *
  * @retval None
  */
void TaskExecuted(eMAIN_Backround_Task_Id_t eMAIN_Backround_Task_Id)
{
	__disable_irq();
	TaskExecutionRequested &= ~(1 << eMAIN_Backround_Task_Id);
	__enable_irq();

	return;
}

/**
  * @brief  Background task
  *
  * @note
  *
  * @param  None
  *
  * @retval None
  */
void Background (uint32_t event_mask)
{
	if(TaskExecutionRequested & event_mask & (1<< eMAIN_Main_Hrs_Evt_Id))
	{
		TaskExecuted(eMAIN_Main_Hrs_Evt_Id);
		ble_hrs_evt_process();
	}

	if(TaskExecutionRequested & event_mask & (1<< eMAIN_Main_HeartRate_Id))
	{
		TaskExecuted(eMAIN_Main_HeartRate_Id);
		heart_rate_meas_timeout_handler();
	}

	if(TaskExecutionRequested & event_mask & (1<< eMAIN_Main_Battery_Id))
	{
		TaskExecuted(eMAIN_Main_Battery_Id);
		battery_level_update();
	}

	if(TaskExecutionRequested & event_mask & (1<< eMAIN_Main_ConnUpdate_Id))
	{
		TaskExecuted(eMAIN_Main_ConnUpdate_Id);
		pfTimerBLECallBack();
	}
#if(APP_HTS == 1)
	if(TaskExecutionRequested & event_mask & (1<< eMAIN_Main_ADC_Data_Processing_Id))
	{
		TaskExecuted(eMAIN_Main_ADC_Data_Processing_Id);
		ProcessADCdata();
	}
#endif
	if(TaskExecutionRequested & event_mask & (1<< eMAIN_Main_GAP_Disc_Evt_Id))
	{
		TaskExecuted(eMAIN_Main_GAP_Disc_Evt_Id);
		ble_bondmngr_bonded_centrals_store();
    advertising_start();
	}
#if(APP_L2CAP_TX_TEST == 1)
	if(TaskExecutionRequested & event_mask & (1<< eMAIN_Main_L2CAP_CID_Register_Id))
	{
		TaskExecuted(eMAIN_Main_L2CAP_CID_Register_Id);
		sd_ble_l2cap_cid_register(L2CAP_HEADER_CID);
		blocking_resp_wait();

        TaskExecutionRequest(eMAIN_Main_L2CAP_TX_Packet_Id);
	}

	if(TaskExecutionRequested & event_mask & (1<< eMAIN_Main_L2CAP_TX_Packet_Id))
	{
		FreeBufferCount--;
		if(FreeBufferCount == 0)
		{
			TaskExecuted(eMAIN_Main_L2CAP_TX_Packet_Id);
		}

		sd_ble_l2cap_tx(GetConnectionHandle(), &l2cap_header , L2CAPPayloadBuffer);
		blocking_resp_wait();
	}
#endif
#if (APP_LEDBUTTON == 1)	
	if(TaskExecutionRequested & event_mask & (1<< eMAIN_Main_Button1_Evt_Id))
	{
		TaskExecuted(eMAIN_Main_Button1_Evt_Id);
		ble_button_process((uint8_t)BUTTON_1);
	}
	
	if(TaskExecutionRequested & event_mask & (1<< eMAIN_Main_Button2_Evt_Id))
	{
		TaskExecuted(eMAIN_Main_Button2_Evt_Id);
		ble_button_process((uint8_t)BUTTON_2);
	}

	if(TaskExecutionRequested & event_mask & (1<< eMAIN_Main_Button3_Evt_Id))
	{
		TaskExecuted(eMAIN_Main_Button3_Evt_Id);
		ble_button_process((uint8_t)BUTTON_3);
	}
#endif	
	/**
	 * Power management
	 */
	__disable_irq();
	if((TaskExecutionRequested & event_mask) == 0)
	{
		LPM_Enter_Mode();
	}
	__enable_irq();

        return;
}

/**
  * @brief  Actions to execute when going out of stop mode
  *
  * @note
  *
  * @param  None
  *
  * @retval None
  */
USED void LPM_ExitStopMode(void)
{
#if (HCLK_32MHZ == 1)
	__HAL_RCC_PLL_ENABLE();										/**< Enable PLL */
	while(__HAL_RCC_GET_FLAG(RCC_FLAG_PLLRDY) == RESET);		/**< Wait till PLL is ready */
	__HAL_APP_RCC_SET_SYSCLK_SRC(RCC_CFGR_SW_PLL);				/**< Select PLL as system clock source */
	while (__HAL_RCC_GET_SYSCLK_SOURCE() != RCC_CFGR_SWS_PLL);	/**< Wait till PLL is used as system clock source */
#endif

	 return;
}

/**
  * @brief  Timer notification from the timer server
  *
  * @note	This is the location where the context may be changed to process the timer callback
  *
  * @param  eTimerModuleID: Identifier of the process owner of the callback
  *
  * @param  pfTimerCallBack: Callback to execute
  *
  * @retval None
  */
void TIMER_Notification(eTIMER_ModuleID_t eTimerModuleID, pf_TIMER_TimerCallBack_t pfTimerCallBack)
{
	switch(eTimerModuleID)
	{
		case eTimerModuleID_BLE:
			   pfTimerBLECallBack = pfTimerCallBack;
			   TaskExecutionRequest(eMAIN_Main_ConnUpdate_Id);
			   break;

		case eTimerModuleID_HeartRate:
			   TaskExecutionRequest(eMAIN_Main_HeartRate_Id);
			   break;

		case eTimerModuleID_Battery:
			   TaskExecutionRequest(eMAIN_Main_Battery_Id);
			   break;

		case eTimerModuleID_Interrupt:
			if(pfTimerCallBack != 0)
			{
				pfTimerCallBack();
			}
			break;

		default:
			if(pfTimerCallBack != 0)
			{
				pfTimerCallBack();
			}
			break;
	}

	return;
}

/**
  * @brief  This function handles all transfer over UART to the BLE module
  *
  * @note	This is the location where the context may be changed to process the message
  *
  * @param  pf_PhyDriverEvent_Handler: Callback
  *
  * @param  pPhyDriverEvent:	Type of notification
  *
  * @retval None
  */
void LPPUART_Msg_Handler(pf_LPPUART_PhyDriverEvent_Handler_t pf_PhyDriverEvent_Handler, sLPPUART_PhyDriverEvent_t pPhyDriverEvent)
{
	pf_PhyDriverEvent_Handler(pPhyDriverEvent);

	return;
}

/**
  * @brief  This function handles the user button
  *
  * @note
  *
  * @param  pf_Main_UserButtonHandler: User button Callback
  *
  * @retval None
  */
void UserButtonHandlerRegister(pf_Main_UserButtonHandler_t pf_Main_UserButtonHandler)
{
	pf_UserButtonHandler = pf_Main_UserButtonHandler;

	return;
}

#if (APP_HTS == 1)
/**
  * @brief  Regular conversion complete callback in non blocking mode
  * @param  hadc: pointer to a ADC_HandleTypeDef structure that contains
  *         the configuration information for the specified ADC.
  * @retval None
  */
void HAL_ADC_ConvCpltCallback(ADC_HandleTypeDef* hadc)
{
	  TaskExecutionRequest(eMAIN_Main_ADC_Data_Processing_Id);

	  return;
}


/**
  * @brief  Return the temperature measured from the ADC
  *
  * @note
  *
  * @param  None
  *
  * @retval uint32_t: Temperature measured
  */
uint32_t getTemperatureValue(void)
{
	return (uint32_t)CalibratedTemperatureData;
}

/**
  * @brief  Start acquisition from ADC
  *
  * @note
  *
  * @param  None
  *
  * @retval None
  */
void acquireTemperatureData(void)
{
	HAL_ADC_Start_DMA(&hadc, (uint32_t*)&RawTemperatureData, 1);
	LPM_Mode_Request(eLPM_ADC_Temp_Measurement, eLPM_Mode_Sleep);
}

/**
  * @brief  DMA interrupt hanlder for ADC
  *
  * @note
  *
  * @param  None
  *
  * @retval None
  */
void App_DMA_ADC_Interrupt_Handler(void)
{
	HAL_DMA_IRQHandler(hadc.DMA_Handle);
}

#endif

#if (APP_LEDBUTTON == 1)
/**@brief Function  Led services.
 *
 * @param[in]   Led_signal_level  Requested ON/OFF led.
 */
void led_value(uint8_t led_value)
{
#if(APP_LCD == 1)
  LCD_SetTextColor(LCD_COLOR_BLUE);	/**< Set Text color */
  bmpcounter = 	led_value;
#endif

	if((led_value&LED1_VALUE)==LED1_VALUE )
	{
		wavetek_nRF_led_on(Wavetek_nRF_LED1_Id);
#if(APP_LCD == 1)
		LCD_DisplayStringLine(LCD_LINE_16, (uint8_t*)"   LED1 : ON   ");
#endif	
	}
	else
	{
		wavetek_nRF_led_off(Wavetek_nRF_LED1_Id);
#if(APP_LCD == 1)
		LCD_DisplayStringLine(LCD_LINE_16, (uint8_t*)"   LED1 : OFF    ");
#endif		
	}

	if((led_value&LED2_VALUE)==LED2_VALUE )
	{
		wavetek_nRF_led_on(Wavetek_nRF_LED2_Id);
#if(APP_LCD == 1)
		LCD_DisplayStringLine(LCD_LINE_17, (uint8_t*)"   LED2 : ON   ");
#endif		
	}
	else
	{
		wavetek_nRF_led_off(Wavetek_nRF_LED2_Id);
#if(APP_LCD == 1)
		LCD_DisplayStringLine(LCD_LINE_17, (uint8_t*)"   LED2 : OFF    ");
#endif		
	}

	if((led_value&LED3_VALUE)==LED3_VALUE )
	{
		wavetek_nRF_led_on(Wavetek_nRF_LED3_Id);
#if(APP_LCD == 1)
		LCD_DisplayStringLine(LCD_LINE_18, (uint8_t*)"   LED3 : ON   ");
#endif		
	}
	else
	{
		wavetek_nRF_led_off(Wavetek_nRF_LED3_Id);
#if(APP_LCD == 1)
		LCD_DisplayStringLine(LCD_LINE_18, (uint8_t*)"   LED3 : OFF   ");
#endif		
	}
}
#endif

#if(APP_L2CAP_TX_TEST == 1)
/**
  * @brief  Update buffer count
  *
  * @note
  *
  * @param  None
  *
  * @retval None
  */
void UpdateL2CAPBufferCount(uint8_t BufferCount)
{
	FreeBufferCount += BufferCount;

	return;
}
#endif

/**
  * @brief  This function is empty to avoid starting the SysTick Timer
  * @param  None
  * @retval None
  */
HAL_StatusTypeDef HAL_InitTick(uint32_t TickPriority)
{
  return (HAL_OK);
}


/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/
